from functools import wraps
import redis
import time


class RateLimitExceededException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

class RateLimiter:
    def __init__(self, redis_host='localhost', redis_port=6379, redis_db=0):
        self.redis_client = redis.Redis(host=redis_host, port=redis_port, db=redis_db)

    def limit(self, limit, period):
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                info = args[1]
                client_ip = info.context['request'].client.host  # Adjust according to actual context setup
                key = f"{func.__module__}:{func.__name__}:{client_ip}"
                current_window = int(time.time() / period)
                key = f"{key}:{current_window}"
                current_count = self.redis_client.get(key)

                if current_count is None or int(current_count) < limit:
                    self.redis_client.incr(key)
                    self.redis_client.expire(key, period)
                    return func(*args, **kwargs)
                else:
                    raise RateLimitExceededException("Rate limit exceeded")
            return wrapper
        return decorator
